<?php

namespace Drupal\web_page_archive\Form;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Field\FieldItemList;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\web_page_archive\Plugin\FileStorageTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;

/**
 * Generates a report for a web page archive run.
 */
class GenerateReportForm extends FormBase {

  use FileStorageTrait;

  /**
   * Constructs a new GenerateReportForm.
   *
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   Date formatter service.
   */
  public function __construct(DateFormatterInterface $date_formatter) {
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'web_page_archive_generate_report';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $web_page_archive_run_revision = NULL) {
    $this->runRevision = $web_page_archive_run_revision;

    $form['intro'] = [
      '#markup' => $this->t('Select the criteria for your report:'),
    ];

    $options = ['csv' => $this->t('CSV')];
    $form['report_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Report Type'),
      '#options' => $options,
      '#default_value' => 'csv',
    ];

    $form['button'] = [
      '#prefix' => '<div class="generate-report-button">',
      '#type' => 'submit',
      '#value' => $this->t('Generate Report'),
      '#suffix' => '</div>',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $report_type = $form_state->getValue('report_type');
    if ($report_type == 'csv') {
      return $this->submitFormCsv($form, $form_state);
    }

    throw new \Exception('Invalid form type');
  }

  /**
   * Generates a CSV report.
   *
   * @param array &$form
   *   Form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state object.
   */
  public function submitFormCsv(array &$form, FormStateInterface $form_state) {
    $captured = $this->runRevision->getCapturedArray();
    $basename = $this->reportTitle($this->runRevision);
    $csv_file = $this->getUniqueFileName(NULL, NULL, $basename, 'tmp', 'csv');
    $fh = fopen($csv_file, 'w');
    $first = TRUE;
    foreach ($this->parseCapturedList($captured) as $captured) {
      if ($first) {
        fputcsv($fh, array_keys($captured));
        $first = FALSE;
      }
      fputcsv($fh, array_values($captured));
    }
    fclose($fh);

    // Generate response.
    clearstatcache(FALSE, $csv_file);
    $response = new BinaryFileResponse($csv_file);
    $response->trustXSendfileTypeHeader();
    $response->headers->set('Content-Type', 'application/csv');
    $response->setContentDisposition(
      ResponseHeaderBag::DISPOSITION_ATTACHMENT,
      $this->fileSystem->basename($csv_file)
    );
    $response->deleteFileAfterSend(TRUE);
    $response->send();

    exit(0);
  }

  /**
   * Parses the captured array for individual capture results.
   */
  private function parseCapturedList(FieldItemList $captured) {
    foreach ($captured as $capture) {
      $unserialized = unserialize($capture->getString());
      if (get_class($unserialized['capture_response']) !== '__PHP_Incomplete_Class') {
        yield $unserialized['capture_response']->getReportData();
      }
    }
  }

  /**
   * Generates the title for the report page.
   */
  public function reportTitle($web_page_archive_run_revision) {
    return $this->t('@label Report - @date', [
      '@label' => $web_page_archive_run_revision->label(),
      '@date' => $this->dateFormatter->format($web_page_archive_run_revision->getRevisionCreationTime(), 'custom', 'Y-m-d H:i:s'),
    ]);
  }

}
